/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Copyright: 2012 Labo A.L
 * Authors:
 *   Aaron Lewis <the.warl0ck.1989@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMovie>
#include <QSortFilterProxyModel>
#include <QStandardItemModel>
#include <QMainWindow>
#include <QClipboard>
#include <QAction>
#include <QSettings>
#include <QMessageBox>
#include <QKeyEvent>

#include "provider.h"

#include "qmpwidget.h"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    
private slots:

    void on_stopButton_clicked();

    void on_playPauseButton_clicked();

    void on_muteButton_toggled(bool checked);

protected:
    void keyPressEvent(QKeyEvent *e);
    void closeEvent(QCloseEvent *);

private:
    Ui::MainWindow *ui;
    int m_vol, m_completed_cnt;

    QSortFilterProxyModel *m_filterModel;
    QStandardItemModel *m_model;

    QList<Provider*> m_providers;

private slots:
    void copyDownloadURL ();

    void loadCompleted ();
    void mplayerStateChanged (int s);
    void update_volume_icon();
    void on_actionRefresh_triggered();
    void on_actionToggleView_triggered(bool checked);
    void on_programeListView_doubleClicked(const QModelIndex &index);
    void on_subtitleFilter_textChanged(const QString &arg1);
    void on_searchTextLine_textChanged(const QString &arg1);
    void on_lookForward_clicked();
    void on_lookBackward_clicked();
    void on_searchTextLine_returnPressed();
    void on_actionAbout_Qt_triggered();
    void on_actionAbout_Me_triggered();
    void on_actionQuit_Q_triggered();
};

#endif // MAINWINDOW_H
