/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Copyright: 2012 Labo A.L
 * Authors:
 *   Aaron Lewis <the.warl0ck.1989@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */
#include "provider.h"

Provider::Provider(QStandardItemModel *model, QObject *parent) :
    QObject(parent),
    m_model (model),
    m_nam (new QNetworkAccessManager (this)),
    m_reply (NULL),
    m_playOptions (None)
{
}

Provider::~Provider()
{

}

void Provider::addItem (const QString &icon, const QString &url,
                        const QString &name, const QString &subtitle)
{
    m_model->appendRow(new ProviderItem (url, name, icon, subtitle));
}

//////
TedProvider::TedProvider(QStandardItemModel *model, QObject *parent):
    Provider(model, parent)
{
    m_icon = ":/images/ted.jpg";
    connect (m_nam, SIGNAL(finished(QNetworkReply*)), this, SLOT(finished(QNetworkReply*)));
}

TedProvider::~TedProvider()
{
    if ( m_reply != NULL )
        m_reply->abort();
}

void TedProvider::start()
{
    if ( m_reply != NULL )
        m_reply->abort();
    m_reply = m_nam->get(QNetworkRequest(QUrl::fromEncoded("http://www.ted.com/talks/rss")));
}

void TedProvider::finished(QNetworkReply *reply)
{
    if ( reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() != 200 )
    {
        qDebug() << "TedProvider: ERROR loading contents";
        return;
    }

    QDomDocument doc; doc.setContent(reply->readAll());
    QDomNode channel = doc.documentElement().firstChild();
    QString videoTitle , videoUrl;

    if ( ! channel.isNull() )
    {
        QDomNode item = channel.firstChild();
        while ( ! item.isNull() )
        {
            videoTitle.clear(); videoUrl.clear();

            /* Scan for details for each item */
            QDomElement detail = item.firstChild().toElement();
            while ( ! detail.isNull() )
            {
                if ( detail.tagName() == "title" )
                {
                    videoTitle = detail.text().remove( QRegExp("^TED:\\s+") );
                }

                if ( detail.tagName() == "media:content" )
                {
                    videoUrl = detail.attribute("url");
                }

                /* No other tags needed , break the loop */
                if ( ! videoTitle.isEmpty() && ! videoUrl.isEmpty() )
                {
                    addItem(m_icon, videoUrl, videoTitle);
                    break;
                }

                detail = detail.nextSibling().toElement();
            }

            item = item.nextSibling();
        }
    }

    emit loaded();

}

SciAmericanProvider::SciAmericanProvider (QStandardItemModel *model, QObject *parent):
    Provider(model, parent)
{
    m_icon = ":/images/sci.jpg";

    page.settings()->setAttribute(QWebSettings::JavascriptEnabled , false);
    page.settings()->setAttribute(QWebSettings::AutoLoadImages , false);
    page.settings()->setAttribute(QWebSettings::JavaEnabled , false);

    connect (&page, SIGNAL(loadFinished(bool)), this, SLOT(fini(bool)));
}

SciAmericanProvider::~SciAmericanProvider()
{

}

#define SCIAMERICAN_BASEURL "http://www.scientificamerican.com/podcast/podcasts.cfm?id=60-second-science"
void SciAmericanProvider::start()
{
    page.mainFrame()->load(QString(SCIAMERICAN_BASEURL));
}

void SciAmericanProvider::fini(bool ok)
{
    if ( ! ok )
    {
        qDebug() << "SciAmericanProvider: ERROR loading contents";
        return;
    }

    QWebPage *page = qobject_cast<QWebPage*> (sender());

    const QString & url = page->mainFrame()->url().toString();

    if ( url == SCIAMERICAN_BASEURL )
    {
        QString videoTitle , videoUrl;

        QWebElementCollection lis = page->mainFrame()->documentElement().findAll("li");
        foreach (const QWebElement & li , lis) if ( li.classes().contains("message_box") )
        {

            QWebElementCollection aLinks = li.findAll("a");
            foreach (const QWebElement & a , aLinks)
            {
                videoTitle = a.toPlainText();
                videoUrl = a.attribute("href");

                if ( ! videoTitle.isEmpty() && videoUrl.contains("episode.cfm") )
                {
                    QStandardItem *item = new QStandardItem (QIcon(":/images/loading.gif"), videoTitle);
                    m_model->appendRow(item);

                    mapping[videoUrl] = item;

                    QWebPage *page = new QWebPage;
                    page->settings()->setAttribute(QWebSettings::JavascriptEnabled , false);
                    page->settings()->setAttribute(QWebSettings::AutoLoadImages , false);
                    page->settings()->setAttribute(QWebSettings::JavaEnabled , false);

                    connect (page, SIGNAL(loadFinished(bool)), this, SLOT(fini(bool)));
                    page->mainFrame()->load(videoUrl);

                    break;
                }
            }

        }

        emit loaded();

    }

    if ( url.indexOf("http://www.scientificamerican.com/podcast/episode.cfm?id=") == 0 )
    {
        QString mp3Url , transcript;

        const QWebElementCollection & divs = page->mainFrame()->documentElement().findAll("div");
        foreach (const QWebElement & div , divs) if ( div.attribute("id") == "articleContent" )
        {
            bool found = false;
            const QWebElementCollection & paragraphs = div.findAll("p");
            foreach (const QWebElement & p , paragraphs)
            {
                const QString & xml = p.toInnerXml();
//                if ( xml.contains("<strong>Read More") )
//                {
//                    found = true;
//                    continue;
//                }
//
//                if ( ! found )
//                    continue;

                transcript += xml;
                transcript.append("<br/>");
            }
        }

        const QWebElementCollection & aLinks = page->mainFrame()->documentElement().findAll("a");
        foreach (const QWebElement & a , aLinks)
        {
            if ( a.attribute("href").indexOf("http://www.scientificamerican.com/podcast/podcast.mp3") == 0 )
            {
                mp3Url = a.attribute("href");
                break;
            }
        }

        if ( transcript.isEmpty() )
        {
            transcript = tr("No transcript ? Possibly bug, please report to "
                            "<b>Aaron Lewis (the.warl0ck.1989@gmail.com)</b>");
        }

        QStandardItem *item = mapping.value(url);
        item->setIcon(QIcon(":/images/sci.jpg"));
        item->setData(mp3Url, Qt::UserRole);
        item->setData(transcript, Qt::UserRole + OFFSET_TRANSCRIPT);

        // delete this
        page->deleteLater();
    }

}
