/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Copyright: 2012 Labo A.L
 * Authors:
 *   Aaron Lewis <the.warl0ck.1989@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */
#ifndef PROVIDER_H
#define PROVIDER_H

#include <QObject>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QNetworkRequest>
#include <QStandardItem>
#include <QDomDocument>
#include <QDebug>
#include <QStandardItemModel>
#include <QIcon>
#include <QWebPage>
#include <QWebFrame>
#include <QWebElement>
#include <QWebElementCollection>

#define OFFSET_TRANSCRIPT 1

class ProviderItem: public QStandardItem
{
public:
    ProviderItem (const QString & url, const QString & name,
                  const QString & icon, const QString & subtitle = QString ())
    {
        setText(name);
        setIcon(QIcon(icon));
        setData(url, Qt::UserRole);

        if ( ! subtitle.isEmpty() )
            setData(subtitle, Qt::UserRole + OFFSET_TRANSCRIPT);
    }

};

class Provider: public QObject
{
    Q_OBJECT
public:
    enum PlayOptions {
        None,
        HasTranscipts
    };

    explicit Provider(QStandardItemModel *model, QObject *parent = 0);
    ~Provider();

    /*!
        \brief start loading of network resources
    */
    virtual void start() {}

protected:
    QStandardItemModel *m_model;
    QNetworkAccessManager *m_nam;
    QNetworkReply *m_reply;
    QString m_text, m_icon;
    PlayOptions m_playOptions;

    void addItem (const QString & icon, const QString & url,
                  const QString & name, const QString & subtitle = QString());
    
signals:
    /*! all resources loaded */
    void loaded ();

public slots:

private slots:
    virtual void finished (QNetworkReply *reply) { Q_UNUSED (reply) }

};

class TedProvider: public Provider
{
    Q_OBJECT
public:
    explicit TedProvider (QStandardItemModel *model, QObject *parent = 0);
    ~TedProvider();
    void start();
private slots:
    void finished (QNetworkReply *reply);
};

class SciAmericanProvider: public Provider
{
    Q_OBJECT
public:
    explicit SciAmericanProvider (QStandardItemModel *model, QObject *parent = 0);
    ~SciAmericanProvider();
    void start();
private slots:
    void fini (bool ok);
private:
    QWebPage page;
    QMap<QString,QStandardItem*> mapping;
};

#endif // PROVIDER_H
